/* Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _SYS_UCONTEXT_H
#define _SYS_UCONTEXT_H	1

#include <features.h>
#include <signal.h>

/* We need the signal context definitions even if they are not used
   included in <signal.h>.  */
#include <bits/sigcontext.h>

/* We also need the definition of the userlevel data representation
   for the register contexts.  */
#include <sys/user.h>


/* Type for general register.  */
typedef int greg_t;

/* Number of general registers.  */
#define NGREG	19

/* Container for all general registers.  */
typedef greg_t gregset_t[NGREG];

#ifdef __USE_GNU
/* Number of each register is the `gregset_t' array.  */
enum
{
  GS = 0,
# define GS	GS
  FS,
# define FS	FS
  ES,
# define ES	ES
  DS,
# define DS	DS
  EDI,
# define EDI	EDI
  ESI,
# define ESI	ESI
  EBP,
# define EBP	EBP
  ESP,
# define ESP	ESP
  EBX,
# define EBX	EBX
  EDX,
# define EDX	EDX
  ECX,
# define ECX	ECX
  EAX,
# define EAX	EAX
  TRAPNO,
# define TRAPNO	TRAPNO
  ERR,
# define ERR	ERR
  EIP,
# define EIP	EIP
  CS,
# define CS	CS
  EFL,
# define EFL	EFL
  UESP,
# define UESP	UESP
  SS
# define SS	SS
};
#endif

/*
   Pentium III FXSR, SSE support
	Gareth Hughes <gareth@valinux.com>, May 2000

   The FPU state data structures have had to grow to accomodate the
   extended FPU state required by the Streaming SIMD Extensions.
   There is no documented standard to accomplish this at the moment. */

/* Definitions taken from the kernel headers.  */
struct _libc_fpreg
{
  unsigned short int significand[4];
  unsigned short int exponent;
};

struct _libc_fpxreg
{
  unsigned short int significand[4];
  unsigned short int exponent;
  unsigned short int padding[3];
};

struct _libc_xmmreg
{
  unsigned long int element[4];
};

struct _libc_fpstate_fsave
{
  unsigned long int cw;
  unsigned long int sw;
  unsigned long int tag;
  unsigned long int ipoff;
  unsigned long int cssel;
  unsigned long int dataoff;
  unsigned long int datasel;
  struct _libc_fpreg _st[8];
  unsigned long int status;
};

struct _libc_fpstate_fxsave
{
  unsigned short int cw;
  unsigned short int sw;
  unsigned short int tag;
  unsigned short int opcode;
  unsigned long int ipoff;
  unsigned long int cssel;
  unsigned long int dataoff;
  unsigned long int datasel;
  unsigned long int mxcsr;
  unsigned long int reserved;
  struct _libc_fpxreg _st[8];
  struct _libc_xmmreg _xmm[8];
  unsigned long int padding[56];
  unsigned long int status;
};

struct _libc_fpstate
{
  union
  {
    struct _libc_fpstate_fsave fsave;
    struct _libc_fpstate_fxsave fxsave;
  } fpregs;
  unsigned long int magic;	/* FPU save format: 0 = FSAVE, else FXSAVE */
};

/* Structure to describe FPU registers.  */
typedef struct _libc_fpstate *fpregset_t;

/* Context to describe whole processor state.  */
typedef struct
  {
    gregset_t gregs;
    /* Due to Linux's history we have to use a pointer here.  The SysV/i386
       ABI requires a struct with the values.  */
    fpregset_t fpregs;
    unsigned long int oldmask;
    unsigned long int cr2;
  } mcontext_t;

/* Userlevel context.  */
typedef struct ucontext
  {
    unsigned long int uc_flags;
    struct ucontext *uc_link;
    stack_t uc_stack;
    mcontext_t uc_mcontext;
    __sigset_t uc_sigmask;
     /* Maintain binary compatibility by only using the old-style FPU save
	format, rather than the full FXSR-compatibile format, here. */
    struct _libc_fpstate_fsave __fpregs_mem;
  } ucontext_t;

#endif /* sys/ucontext.h */
